/' (c) 2022 cepstrum.co.jp           '/
/' This is not free software.        '/
'
' wavwform plot program, 16bit, 48kHz, monoral
'
' GUI program, command line option : fbc -s gui
'
 
#include "portaudio.bi"
#include "graphics.bi"

#define FFTLEN  2048
dim ar(FFTLEN) as single
dim ai(FFTLEN) as single
#include "fft.bi"

dim spec(FFTLEN\2) as single

dim shared copybuf(FFTLEN) as single
dim shared plot_busy       as integer = 0
dim shared frame_count     as integer = 0

#define FS        48000
#define PA_BUFLEN 1

type User_Data_Type
  dummy as any ptr
end type

dim user_data as User_Data_Type

dim stream    as any ptr 
dim errcode   as integer<32>

dim inchar    as string
dim keycode   as integer
dim i         as integer
dim plotx     as integer
dim ploty     as integer
dim zploty    as integer 
dim yoffset   as single = 100

dim mouse_x   as integer
dim mouse_y   as integer
dim buttons   as integer
dim zbuttons  as integer = 0

'==========================================================================

function callbackfunc cdecl(                   _
  pread      as integer<16> ptr,               _
  pwrite     as integer<16> ptr,               _
  rwcount    as uinteger<32>,                  _
  timeInfo   as PaStreamCallbackTimeInfo ptr,  _
  flag       as uinteger<32>,                  _
  puser_data as User_Data_Type ptr             _
) as integer

  static xbuf(FFTLEN) as single
  static init_flag    as integer = 1
  static ix_xbuf      as integer = 0
  dim    i_ad         as integer<16>
  dim    i_da         as integer<16>
  dim    f_ad         as single
  dim    f_da         as single
  dim    x            as single
  dim    i            as integer

  if init_flag=1 then
    init_flag=0
    for i=0 to FFTLEN-1 : xbuf(i)=0.0 : next
    return 0
  end if

  i_ad=*pread
  f_ad=i_ad/32768.0
  x=f_ad

  xbuf(ix_xbuf)=x
  ix_xbuf=(ix_xbuf+1) mod FFTLEN

  if plot_busy=0 then
    for i=0 to FFTLEN-1 : copybuf(i)=xbuf((ix_xbuf+i) mod FFTLEN) : next
  end if

  frame_count=frame_count+1
   
  return 0

end function

'==========================================================================

sub apply_hamwin(x() as single, winlen as integer)
  dim i as integer

  for i=0 to winlen-1 : x(i)=x(i)*(0.54-0.46*cos(2.0*M_PI*i/(winlen-1.0))) : next
end sub

'==========================================================================

windowtitle " [cepstrum.co.jp]"
screenres 1024+1, 400+1

errcode=Pa_Initialize()

errcode=Pa_OpenDefaultStream(  _ 
  @stream,                     _  ' PaStream ptr ptr        
  1,                           _  ' input channels          
  0,                           _  ' output channels       
  8,                           _  ' sample format (paInt16)  
  FS,                          _  ' Fs                   
  PA_BUFLEN,                   _  ' frames per buffer     
  @callbackfunc,               _  ' callback function  
  @user_data                   _  ' user data      
) 

for i=0 to FFTLEN-1
  ar(i)=0.0
  ai(i)=0.0
next

fft(ar(), ai(), -1, 1)
plot_busy=0
errcode=Pa_StartStream(stream)
do

  if g_lbutton then : yoffset=yoffset-10 : end if   
  if g_rbutton then : yoffset=yoffset+10 : end if  
  if yoffset<-100  then : yoffset=-100  : end if
  if 200.0<yoffset then : yoffset=200.0 : end if

  plot_busy=1
  for i=0 to FFTLEN-1
    ar(i)=copybuf(i)
    ai(i)=0.0
  next
  plot_busy=0

  apply_hamwin(ar(), FFTLEN)

  fft(ar(), ai(), -1, 0)
  for i=0 to FFTLEN\2-1
    spec(i)=ar(i)*ar(i)+ai(i)*ai(i)    
    spec(i)=10.0*log10(spec(i)+1e-19)   
  next
  
  g_fillbox(0, 0, 1024+1, 400+1, BLACK_COLOR)

  for i=0 to FFTLEN\2-1
    plotx=i
    ploty=5*spec(i)+yoffset
    if i=0 then : zploty=ploty : end if
    g_line(plotx, zploty, plotx+1, ploty, GREEN_HIGH_COLOR)
    zploty=ploty
  next
  
  g_tofu(4, 16, 42,BLACK_COLOR)
  g_string(4, 16, "click mouse button to change Y-axis offset", YELLOW_COLOR)
  g_tofu(4, 4, 29,BLACK_COLOR)
  g_string(4, 4, "hit ctrl-C or ESC key to exit", YELLOW_COLOR)

  inchar=inkey$
  keycode=asc(inchar)
  if keycode=3 or keycode=27 then           ' if ctrl-C or ESC
    errcode=Pa_StopStream(stream)
    errcode=Pa_CloseStream(stream)
    Pa_Terminate()
    '-------------------------------
    open "out.txt" for output as #1
    for i=0 to FFTLEN/2-1 : print #1, i, spec(i) : next
    close #1
    '-------------------------------
    system
  end if

  sleep(100)            ' sleep 100ms

loop


